#!/usr/bin/python

import sys,os,time,socket,signal,struct,zipfile

def getVersionString(s):
    cmd=5
    index=0
    length=8
    addr=0
    s.send(struct.pack("<BBHL",cmd,index,length,addr))
    resp = s.recv(1024)
    fmt = ">LLLLLL64s6sLBBHBBBB"
    devIpAddr, defRoute, srvIpAddr, ipMask, availMemAddr, availMemLength, version, macAddr, \
    modelId, archId, numPorts, progId, boardRev, bootRevMajor, bootRevMinor, numApps \
      = struct.unpack(fmt,resp[:struct.calcsize(fmt)])
    return version[:version.index('\x00')]

def getArch((host,port),useIPv6=0):

    if useIPv6:
        s = socket.socket(socket.AF_INET6, socket.SOCK_STREAM)
    else:
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.settimeout(15)
    try:
        s.connect((host,port))
    except socket.timeout:
        return "network connection timeout";
    except (OSError, socket.error) as e:
        return "Error opening connection: " + str(e)

    cmd=5
    index=0
    length=8
    addr=0

    try:
        s.send(struct.pack("<BBHL",cmd,index,length,addr))
        resp = s.recv(1024)
    except socket.timeout:
        return "network connection timeout";
    except (OSError, socket.error) as e:
        return "error requesting device ID info: " + str(e)

    return ord(resp[98])


# return error message if failure otherwise return None
def uploadFile((host,port),filedata,doreset,callback,useIPv6=0):
    filesize = len(filedata)

    if doreset:
        if useIPv6:
            s = socket.socket(socket.AF_INET6, socket.SOCK_STREAM)
        else:
            s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        callback(0,"opening connection")
        s.settimeout(15)
        try:
            s.connect((host,port))
        except socket.timeout:
            return("network connection timeout");
        except (OSError, socket.error) as e:
            return "Error opening connection: " + str(e)
        callback(0,"sending reset command")
        try:
            s.send(struct.pack("<BBHL",13,0,8,0))
            s.close()   # reset won't happen until conneciton is closed
        except socket.timeout:
            return("timeout sending reset command");
        except (OSError, socket.error) as e:
            return "Error sending reset command: " + str(e)
        # wait 3 seconds for the unit to reset and come back up
        for i in xrange(6):
            time.sleep(0.5)
            callback((filesize*i)/6)
        callback(filesize)

    if useIPv6:
        s = socket.socket(socket.AF_INET6, socket.SOCK_STREAM)
    else:
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    callback(0,"opening connection")
    s.settimeout(15)
    try:
        s.connect((host,port))
    except socket.timeout:
        return("Timeout opening connection");
    except (OSError, socket.error) as e:
        return "Error opening connection: " + str(e)

    # if 'doreset' then verify we're talking to RedBoot

    if doreset:
        try:
            version = getVersionString(s)
            if not version.startswith('Bootloader'):
                return "Expected to connect to bootloader after reboot but found %s" % version
        except Exception as e:
            return "Failure verifying bootloader is running:\n\n" + str(e)

    try:
        index = 0
        bytecount = 0
        s.settimeout(10)
        while bytecount < filesize:
            data = filedata[bytecount:bytecount+1400]
            length = len(data)
            dest = bytecount
            header = struct.pack("<BBHL",1,index,length+8,dest)
            time.sleep(0.01)
            s.send(header+data)
            bytecount += length
            index += 1
            index &= 0xff
            callback(bytecount, "uploaded %7d of %d bytes" % (bytecount,filesize))
        callback(bytecount, "starting program")
        entry = 0
        s.send(struct.pack("<BBHL",3,0,8,entry))
    except socket.timeout:
        return "Network timeout"
    except OSError as e:
        return "Error uploading data: " + str(e)
    finally:
        s.close()

if sys.hexversion >= 0x030000F0:
    from tkinter import *
    from tkinter.ttk import *
    import tkinter.filedialog as filedialog
else:
    from Tkinter import *
    from ttk import *
    import tkFileDialog as filedialog
    import tkMessageBox as messagebox

class App:
    def __init__(self, master):
        self.hostname = StringVar()
        self.filename = StringVar()
        self.doreset  = IntVar()
        self.useIPv6  = IntVar()
        self.portnum  = StringVar()
        frame = Frame(master)
        frame.columnconfigure(0,weight=0)
        frame.columnconfigure(1,weight=4)
        frame.columnconfigure(2,weight=0)
        frame.columnconfigure(3,weight=0)
        r = 0
        Label(frame, text="Comtrol DeviceMaster Updater").grid(row=r, columnspan=4,pady=5)
        r += 1
        Label(frame, text="Host: ").grid(row=r,column=0,sticky=E,pady=2)
        Entry(frame, width=25, textvariable=self.hostname).grid(row=r,column=1,pady=5,sticky=E+W)
        Label(frame, text="  Port: ").grid(row=r,column=2,pady=2,sticky=E)
        Entry(frame, width=7, textvariable=self.portnum).grid(row=r,column=3,pady=5,sticky=W)
        self.portnum.set("4606")
        r += 1
        Label(frame, text="File: ").grid(row=r,column=0,sticky=E,pady=2)
        Entry(frame, width=50, textvariable=self.filename).grid(row=r,column=1,columnspan=2,pady=5,sticky=E+W)
        Button(frame, text="browse", command=self.browse).grid(row=r,column=3,padx=3,pady=5)
        r += 1
        Checkbutton(frame, text="Use IPv6",command=self.changeIPv6,variable=self.useIPv6).grid(row=r,columnspan=4,pady=5,sticky=W)
        self.useIPv6.set(0)
        r += 1
        Checkbutton(frame, text="Reboot DM unit before upload",variable=self.doreset).grid(row=r,columnspan=3,pady=5,sticky=W)
        self.doreset.set(1)
        Button(frame, text="Help",command=self.help).grid(row=r,column=3,pady=5)
        r += 1
        Button(frame, text="Upload",command=self.upload).grid(row=r,columnspan=4,pady=5)
        frame.grid(padx=10,pady=10,sticky=N+S+E+W)
        top=frame.winfo_toplevel() 
        top.rowconfigure(0, weight=1)
        top.columnconfigure(0, weight=1)

    def help(self):
        mess

    def changeIPv6(self):
        if self.useIPv6.get():
            self.doreset.set(0)

    def browse(self):
        filetypes = [("Comtrol firmware files",".cmtl"),(".bin files",".bin"),(".nxbin files",".nxbin"),("all files",".*")]
        filename = filedialog.askopenfilename(title="Select file to upload", initialdir=os.path.expanduser('_'), filetypes=filetypes)
        self.filename.set(filename)

    def upload(self):
        hostname = self.hostname.get()
        filename = self.filename.get()

        if not (hostname and filename and self.portnum.get()):
            messagebox.showerror("Host/File Required","Hostname or IP-address, Port, and Filename required")
            return

        doreset  = self.doreset.get()
        useIPv6  = self.useIPv6.get()
        portnum  = int(self.portnum.get())

        if portnum <= 0 or portnum > 65535:
            messagebox.showerror("Invalid Port","Invalid port number.  Port must be 1 to 65535")
            return
        try:
            f = open(filename,'rb')
        except OSError as e:
            messagebox.showerror("Error opening file","Error opening file '%s':\n\n%s" % (filename,e))
            return;

        try:
            is_zip = zipfile.is_zipfile(f)
        except Exception as e:
            messagebox.showerror("Error checking file","is_zipfile '%s':\n\n%s" % (filename,e))
            return;
        f.close()

        if is_zip:
            arch = getArch((hostname,portnum),useIPv6)
            if type(arch) == str:
                messagebox.showerror("Upload Failed", "Error reading device ID:\n\n %s" % arch)
                return
            try:
                archsuffix = {3:'.bin', 5:'.nxbin'}
                if arch not in archsuffix:
                    messagebox.showerror("Upload Failed", "Unkown device arch ID (%d)" % arch)
                    return
                zf = zipfile.ZipFile(filename,'r')
                for n in zf.namelist():
                    if n.endswith(archsuffix[arch]):
                        f = zf.open(n,'r')
                        break
                else:
                    messagebox.showerror("Upload Failed", "No binary found for arch ID (%d)" % arch)
                    return                        
            except Exception as e:
                messagebox.showerror("Upload Failed", "Error in zip archive:\n\n %s" % e)
                return
            filedata = f.read()
            f.close()
            zf.close()
        else:
            try:
                f = open(filename,'rb')
            except OSError as e:
                messagebox.showerror("Upload Failed", "Error opening file '%s':\n\n %s" % (filename,e))
                return
            filedata = f.read()
            f.close()

            
        top = Toplevel(root)
        top.geometry("+%d+%d" % (root.winfo_rootx()+100,root.winfo_rooty()+20))

        msg = "uploading %s to %s" % (os.path.basename(filename), hostname)
        Label(top,text=msg).grid(padx=5,pady=5)
        progress = Progressbar(top, orient="horizontal", length=400, mode="determinate", value=0, maximum=len(filedata))
        progress.grid(sticky=E+W,padx=5,pady=5)
        status = StringVar()
        Label(top,textvariable=status).grid(padx=5,pady=5)
        def done():
            top.destroy()
        doneButton = Button(top, text=" OK ",command=done,state=DISABLED,default=ACTIVE)
        doneButton.grid(padx=5,pady=5,sticky=E)
        def callback(bytesWritten,statusMsg=None):
            progress['value'] = bytesWritten
            if statusMsg is not None:
                status.set(statusMsg)
            root.update()
        progress['value'] = 0
        root.update()
        

        error = uploadFile((hostname,portnum),filedata,doreset,callback,useIPv6)
        if error:
            top.destroy()
            messagebox.showerror("Upload Failed",error)
        else:
            status.set("Done")
            doneButton['state'] = NORMAL;
            root.wait_window(top)


    def help(self):
        top = Toplevel(root)
        Label(top,text="Upload Help").grid(padx=5,pady=5)
        def done():
            top.destroy()
        t = Text(top, width=80, height=helptext.count('\n')+1)
        t.insert(INSERT,helptext)
        t['state'] = DISABLED
        t.grid(padx=5,pady=5,sticky=E+W+N+S)
        Button(top, text=" OK ",command=done,default=ACTIVE).grid(padx=5,pady=5,sticky=E)
        top.columnconfigure(0,weight=1)
        top.rowconfigure(1,weight=1)
        root.wait_window(top)

helptext = \
"""

  HOST can be either a hostname to be resolved by the operating system
  or a numerical IP address in standard format (nnn.nnn.nnn.nnn for
  IPv4 or xxxx:xxx::xxx:xxxx for IPv6).

  PORT must be set to the TCP port number to be used.  DeviceMaster
  RTS units always listen on port 4606, so the default value should be
  correct unless a NAT firewall is being used.

  FILE should be the absolute or relative path of the firware binary
  file to be uploaded.

  "Use IPv6" when selected will cause IPv6 may be used instead of
  IPv4.  IPv6 is only supported if the DM RTS unit is running firmware
  version 9.00 or later and has been configured to use IPv6.  If IPv6
  is used, the unit should _not_ be rebooted before upload.

  "Reboot before upload" _must_ be enabled if the DM RTS unit is
  running firmware version 8.xx or older.  If IPv6 is being used (only
  supported by firmware 9.00 or newer), the unit should not be
  rebooted before upload.  If IPv4 is being used with firmware 9.00 or
  newer, the upload can be done either with or without a reboot.

           Firmware        IP Vers    Reboot
       ------------------  -------    ------
        8.xx (or older)       4         Y
        9.xx (or newer)       4       either
        9.xx (or newer)       6         N

"""

logodata=\
"""
R0lGODlhPAA8APUAANwdIdwhJt0lKd4pLt4tMt8xNd81Od86PuAvNeAzNuA2OuA6PuA8QOFBROFF
SeNJTeROUuRQU+RWWuVZXeZfYudiZedmaehmaudrbuhqbuducOhucelxc+l2eep6fOyHie+SlO+Z
mvGjpfGmqPKsrfOvsPOytPS3ufS5u/XCw/fOzvfP0PjP0PjS0/nX2fnY2vvk5fzs7f329v///wAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAACH5BAAAAAAAIf8LSW1hZ2VNYWdpY2sN
Z2FtbWE9MC40NTQ1NQAsAAAAADwAPAAABv7AmXBILBqPMxkohmw6n1CjzCKQMKPY7FNWGSwKk6t2
PJZ1BozFN0xuR2UetFpdsLrvyPN8Xxfj8Xp7fHZ/bnACgol9hWRmAweJioSMWIGRkn6USDIcXpeX
dTCaTTIZcp+RCgURoqNFMhgIqIKQcwcFEK2uSaazvri6mjIXnr6oqqyjpcW+CgvOatDOCgTJjMOn
Cwe1ggoMEA/h4uPiDGx/MhSICgoG7QYGB/Du8SMy9/j5+veFIRMcHDYAHEgwIAYOKnYpXMhwIb4k
+ZJI5BexoRBOBRikmZNG4wGPGhcw+IjC4rJ28uIpSDmvZcsRE2XEnDjz4ZM4G41d6hhSZP5IjUB/
MlBwIdMQOMx0dktgoACCpu4KSFWAgKlTBKrYIRgQwagZAezChn02Taw0lB1QqF3Ltq3btSUSEoGD
oIHduwzu6t3bIG/fBikUxkjBorCKwogRr1jcQsXiFYkbG7VIuXJlFyNMaN7MubPnzsE0xZAAQIDp
06hTqx5wGubCDgXU5BSZqKM2NZAYcDBRorfv38CD9xaRQuYr2NyU7kEQoLlz5wCeB4j+PDqABi42
aYitPJGD7w4agB9PnnyDCqFfZSgAj/07A0zhtYvf3sCHF/hfuNCP38X+/v/x599kr3hgwQYIIogB
BhlooMGCEC6YwYItWGaccUdZpiFlD9B1GFGHG8LRQAQklmhiiRBAEEFgG8agDgFSSYXAU07FSAAB
rlFEE4YYopOBLHvM9tE2EhSkQUEAaWDBBwQ2ogEBxhQwwAAETFkllVcOEAB2yjxpjEcfybbNSKHs
IsMGUPoy5Da23CJBepTAkuYs2yT3xZsNnTmncmWaxMGexiAwAZwKwUGAN3ZGUgB6If4JTSSQqMLo
hkl0QIABsz1TZwEUEMqhpYlus6inlSGl6KSUHrenpKQ2ikganLZKaSlQItBpqlDAUoAFsuI6wwsr
jhIEADs=
"""        

root = Tk()
progname = os.path.basename(sys.argv[0])
progname = os.path.splitext(progname)[0]
root.wm_title(progname)
root.logoimg = PhotoImage(data=logodata)
root.tk.call('wm','iconphoto', root._w, root.logoimg)
app = App(root)
root.mainloop()

